"""
 Copyright (c) 2021 Faisal Thaheem (https://github.com/faisalthaheem/python-stanag-4586-EDA-v1)
 License GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from ctypes import *
from pprint import pprint
from .message_base import MessageBase

MSG200_NULL = b"\x00"*68

#Constants for set zoom field
MSG200_SET_ZOOM_USE_HV_FOV = 0
MSG200_SET_ZOOM_STOP_ZOOM = 1
MSG200_SET_ZOOM_ZOOM_IN = 2
MSG200_SET_ZOOM_ZOOM_OUT = 3

#Constants for Altitude type field
MSG200_ALTITUDE_TYPE_PRESSURE = 0
MSG200_ALTITUDE_TYPE_BARO = 1
MSG200_ALTITUDE_TYPE_AGL = 2
MSG200_ALTITUDE_TYPE_WSG84 = 3

#Constants for set focus field
MSG200_SET_FOCUS_NO_CHANGE = 0
MSG200_SET_FOCUS_FOCUS_CLOSER = 1
MSG200_SET_FOCUS_FOCUS_FARTHER = 2

#Constants for focus type field
MSG200_FOCUS_TYPE_AUTO = 0
MSG200_FOCUS_TYPE_MANUAL = 1

class Message200(BigEndianStructure, MessageBase):
    _pack_ = 1
    _fields_ = [
        ("time_stamp",                          c_double),
        ("vehicle_id",                          c_int32),
        ("cucs_id",                             c_int32),
        ("station_number",                      c_uint32),
        ("set_centreline_azimuth_angle",        c_float),
        ("set_centreline_elevation_angle",      c_float),
        ("set_zoom",                            c_ubyte),
        ("set_horizontal_fov",                  c_float),
        ("set_vertical_fov",                    c_float),
        ("horizontal_slew_rate",                c_float),
        ("vertical_slew_rate",                  c_float),
        ("latitude",                            c_double),
        ("longitude",                           c_double),
        ("altitude",                            c_float),
        ("altitude_type",                       c_ubyte),
        ("set_focus",                           c_ubyte),
        ("focus_type",                          c_ubyte),
      ]
    
    def __new__(cls, byte_buffer=None):
        return cls.from_buffer_copy(byte_buffer)

    def __init__(self, byte_buffer=None):
        pass

    def encode(self):
        return bytes(self)

    def dump(self):
        pprint(self)


"""
 Copyright (c) 2021 Faisal Thaheem (https://github.com/faisalthaheem/python-stanag-4586-EDA-v1)
 License GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from ctypes import *
from pprint import pprint
from .message_base import MessageBase

MSG20000_NULL = b"\x00"*68

MSG20000_PAN_DIRECTION_LEFT = -1
MSG20000_PAN_DIRECTION_NONE = 0
MSG20000_PAN_DIRECTION_RIGHT = 1

MSG20000_TILT_DIRECTION_UP = 1
MSG20000_TILT_DIRECTION_NONE = 0
MSG20000_TILT_DIRECTION_DOWN = -1

class Message20000(BigEndianStructure, MessageBase):
    """Custom Message 20000 for continually Panning and Tilting EO payloads"""
    
    _pack_ = 1
    _fields_ = [
        ("time_stamp",                          c_double),
        ("vehicle_id",                          c_int32),
        ("cucs_id",                             c_int32),
        ("station_number",                      c_uint32),
        ("pan_force",                           c_float),
        ("pan_direction",                       c_byte),
        ("tilt_force",                          c_float),
        ("tilt_direction",                      c_byte),
      ]
    
    def __new__(cls, byte_buffer=None):
        return cls.from_buffer_copy(byte_buffer)

    def __init__(self, byte_buffer=None):
        pass

    def encode(self):
        return bytes(self)

    def dump(self):
        pprint(self)


"""
 Copyright (c) 2021 Faisal Thaheem (https://github.com/faisalthaheem/python-stanag-4586-EDA-v1)
 License GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

class MessageBase:
    def set_string_field(self, field_name, field_max_length, field_value):

        if len(field_value) > field_max_length or len(field_value) == 0:
            return False

        setattr(self, field_name, field_value.encode("ascii"))

        return True

    def get_string_field(self, field_name):
        if not hasattr(self, field_name): 
            return

        return bytes(getattr(self, field_name)).decode('ascii')


"""
 Copyright (c) 2021 Faisal Thaheem (https://github.com/faisalthaheem/python-stanag-4586-EDA-v1)
 License GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from ctypes import *
from pprint import pprint
from .message_base import MessageBase

MSG301_NULL = b"\x00"*63

class Message301(BigEndianStructure, MessageBase):
    _pack_ = 1
    _fields_ = [
        ("time_stamp",                          c_double),
        ("vehicle_id",                          c_int32),
        ("cucs_id",                             c_int32),
        ("vsm_id",                              c_int32),
        ("station_number",                      c_uint32),
        ("eo_ir_type",                          c_char*14),
        ("eo_ir_type_revision_level",           c_ubyte),
        ("eo_vertical_image_dimension",         c_int16),
        ("eo_horizontal_image_dimension",       c_int16),
        ("ir_vertical_image_dimension",         c_int16),
        ("ir_horizontal_image_dimension",       c_int16),
        ("field_of_regard_elevation_min",       c_float),
        ("field_of_regard_elevation_max",       c_float),
        ("field_of_regard_azimuth_min",         c_float),
        ("field_of_regard_azimuth_max",         c_float),
      ]
    
    def __new__(cls, byte_buffer=None):
        return cls.from_buffer_copy(byte_buffer)

    def __init__(self, byte_buffer=None):
        pass

    def encode(self):
        return bytes(self)

    def dump(self):
        pprint(self)

    def get_eo_ir_type(self):
        return self.get_string_field("eo_ir_type")

    def set_eo_ir_type(self, val):
        return self.set_string_field("eo_ir_type", 14, val)


"""
 Copyright (c) 2021 Faisal Thaheem (https://github.com/faisalthaheem/python-stanag-4586-EDA-v1)
 License GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from ctypes import *
from pprint import pprint
from .message_base import MessageBase

MSG20_NULL = b"\x00"*99

class Message20(BigEndianStructure, MessageBase):
    _pack_ = 1
    _fields_ = [
        ("time_stamp",              c_double),
        ("vehicle_id",              c_int32),
        ("cucs_id",                 c_int32),
        ("vsm_id",                  c_int32),
        ("vehicle_id_update",       c_int32),
        ("vehicle_type",            c_ushort),
        ("vehicle_sub_type",        c_ushort),
        ("owning_id",               c_ubyte),
        ("tail_number",             c_char*16),
        ("mission_id",              c_char*20),
        ("atc_call_sign",           c_char*32),
        ("configuration_checksum",  c_ushort),
      ]
    
    def __new__(cls, byte_buffer=None):
        return cls.from_buffer_copy(byte_buffer)

    def __init__(self, byte_buffer=None):
        pass

    def encode(self):
        return bytes(self)

    def dump(self):
        pprint(self)

    def get_tail_number(self):
        return self.get_string_field("tail_number")

    def set_tail_number(self, val):
        return self.set_string_field("tail_number", 16, val)

    def get_mission_id(self):
        return self.get_string_field("mission_id")

    def set_mission_id(self, val):
        return self.set_string_field("mission_id", 20, val)

    def get_atc_call_sign(self):
        return self.get_string_field("atc_call_sign")

    def set_atc_call_sign(self, val):
        return self.set_string_field("atc_call_sign", 32, val)